# -*- coding: utf-8 -*-
import os
import sys
import shutil
import yaml
from distutils.core import setup
from Cython.Build import cythonize


class SoBuilder(object):

    def __init__(self, app_path):
        self.app_path = app_path
        self.base_path = os.path.abspath('.')
        self.build_path = 'build'
        self.build_tmp_path = 'build/tmp'
        if os.path.exists(self.build_path):
            shutil.rmtree(self.build_path)

        py_ver = ''.join(sys.version[0:3].split('.'))
        self.gcc_suffix = '.cpython-{}m-x86_64-linux-gnu.so'.format(py_ver)

    def copy_other_file(self, src_file_path):
        if src_file_path.endswith('__init__.py'):
            if os.path.exists(self.build_path):
                shutil.rmtree(self.build_path)
            raise Exception(print('程序中存在“__init__.py”文件，编译时会出现异常。请删除所有“__init__.py”文件后再编译！'))

        dst_file_path = '{}/{}/{}'.format(self.base_path, self.build_path, src_file_path[len(self.base_path) + 1:])
        dst_path = dst_file_path[:dst_file_path.rfind('/')]
        if not os.path.isdir(dst_path):
            os.makedirs(dst_path)
        shutil.copyfile(src_file_path, dst_file_path)

    def yeild_py(self, path, copy_others=True):
        for file_name in os.listdir(path):
            file_path = os.path.join(path, file_name)
            if os.path.isdir(file_path) and not file_name.startswith('.'):
                for f in self.yeild_py(file_path, copy_others):
                    yield f
            elif os.path.isfile(file_path):
                ext = os.path.splitext(file_name)[1]
                if ext not in ('.pyc', '.pyx'):
                    if ext == '.py' and not file_name.startswith('__'):
                        yield os.path.join(path, file_name)
                    elif copy_others:
                        self.copy_other_file(file_path)
            else:
                pass

    def delete_c_files(self, path):
        for file_name in os.listdir(path):
            file_path = os.path.join(path, file_name)
            if os.path.isdir(file_path) and not file_name.startswith('.'):
                self.delete_c_files(file_path)
            elif os.path.isfile(file_path):
                ext = os.path.splitext(file_name)[1]
                if ext == '.c':
                    os.remove(file_path)
            else:
                pass

    def build_so(self):
        py_files = list(self.yeild_py(os.path.join(self.base_path, self.app_path)))

        try:
            for src_file_path in py_files:
                dst_file_path = '{}/{}/{}'.format(self.base_path, self.build_path,
                                                  src_file_path[len(self.base_path) + 1:])
                idx = dst_file_path.rfind('/')
                dst_path = dst_file_path[:idx]
                py_name = dst_file_path[idx + 1:].split('.')[0]
                setup(ext_modules=cythonize(src_file_path),
                      script_args=['build_ext', '-b', dst_path, '-t', self.build_tmp_path])
                src = dst_path + '/' + py_name + self.gcc_suffix
                dst = dst_path + '/' + py_name + '.so'
                os.rename(src, dst)
        except Exception as e:
            print(str(e))

        self.delete_c_files(os.path.join(self.base_path, self.app_path))
        if os.path.exists(self.build_tmp_path):
            shutil.rmtree(self.build_tmp_path)


def get_model_name():
    try:
        with open('./application.yml', 'r') as f:
            yml = yaml.load(f, Loader=yaml.SafeLoader)
    except:
        print('错误： 模型配置文件application.yml不存在！')
        return None

    try:
        name = yml['model']['name']
    except:
        print('错误： 未指定模型名称！')
        print('请在application.yml文件中编辑修改...')
        return None

    try:
        python = str(yml['python']['version'])
    except:
        print('错误： 未指定Python版本号！')
        print('请在application.yml文件中编辑修改...')
        return None


    if not python.startswith('3.'):
        print('错误： Python版本号必须是3且3.5以上！')
        print('请在application.yml文件中编辑修改...')
        return None

    if not sys.version.startswith(python):
        print('错误： 声明的Python版本号与当前运行环境（{}）不一致！'.format(sys.version[:sys.version.find(' ')]))
        print('请在application.yml文件中编辑修改...')
        return None

    try:
        model_runner = yml['model_runner']['version']
    except:
        print('错误： 未指定model_runner版本号！')
        print('请在application.yml文件中编辑修改...')
        return None

    if model_runner != 'v2':
        print('错误： model_runner版本号必须是“v2”！')
        print('请在application.yml文件中编辑修改...')
        return None

    if not os.path.exists('requirements.txt'):
        print('错误： requirements.txt文件不存在！')
        return None

    return name


def pack_model():
    name = get_model_name()
    if name is None:
        return

    with open('./application.yml', 'r') as f:
        yml = yaml.load(f, Loader=yaml.SafeLoader)

    try:
        build_web = yml['build']['build_web']
    except:
        build_web = True

    try:
        build_so = yml['build']['compile_python_to_so']
    except:
        build_so = False

    os.system('rm -rf ./out')
    os.system('mkdir out')
    os.system('mkdir out/{}'.format(name))
    os.system('cp ./application.yml ./out/{}/'.format(name))
    os.system('cp ./requirements.txt ./out/{}/'.format(name))

    if build_so:
        so_builder = SoBuilder('core')
        so_builder.build_so()
        os.system('mv ./build/core ./out/{}/'.format(name))
        shutil.rmtree('build')
    else:
        os.system('cp -rf ./core ./out/{}/'.format(name))

    if build_web:
        if os.path.exists('./webapp/src'):
            cwd = os.getcwd()
            os.chdir(os.path.join(cwd, 'webapp'))
            if not os.path.exists('./node_modules'):
                os.system('npm install')
            os.system('ng build --prod')
            os.chdir(cwd)
        if os.path.exists('./webapp/www'):
            os.system('mkdir out/{}/webapp'.format(name))
            os.system('cp -rf ./webapp/www ./out/{}/webapp/'.format(name))

    dst_path = './out/{}'.format(name)
    shutil.make_archive(dst_path, 'zip', dst_path)  # 将目标文件夹自动压缩成.zip文件
    shutil.rmtree('./out/{}/'.format(name))
    print('模型打包完成！ 输出位置： ./out/{}.zip'.format(name))


def build_docker():
    name = get_model_name()
    if name is None:
        return

    with open('./application.yml', 'r') as f:
        yml = yaml.load(f, Loader=yaml.SafeLoader)

    try:
        image_name = yml['build']['image_name']
    except:
        print('错误： 未指定docker镜像名称！')
        print('请在application.yml文件中编辑修改...')
        return

    try:
        image_tag = str(yml['build']['tag'])
    except:
        print('未指定docker镜像tag，使用：latest')
        image_tag = 'latest'

    try:
        build_web = yml['build']['build_web']
    except:
        build_web = True

    try:
        build_so = yml['build']['compile_python_to_so']
    except:
        build_so = False

    os.system('rm -rf temp')
    os.system('mkdir temp')
    os.system('cp ./application.yml ./temp')
    os.system('cp ./requirements.txt ./temp')
    os.system('cp ./Dockerfile ./temp')

    if build_so:
        so_builder = SoBuilder('core')
        so_builder.build_so()
        os.system('mv ./build/core ./temp/'.format(name))
        shutil.rmtree('build')
    else:
        os.system('cp -rf ./core ./temp/'.format(name))

    if build_web:
        if os.path.exists('./webapp/src'):
            cwd = os.getcwd()
            os.chdir(os.path.join(cwd, 'webapp'))
            if not os.path.exists('./node_modules'):
                os.system('npm install')
            os.system('ng build --prod')
            os.chdir(cwd)
        if os.path.exists('./webapp/www'):
            os.system('mkdir temp/webapp'.format(name))
            os.system('cp -rf ./webapp/www ./temp/webapp/'.format(name))

    os.system('docker image rm {}:{}'.format(image_name, image_tag))
    os.system('docker build -t {}:{} ./temp'.format(image_name, image_tag))
    os.system('rm -rf temp')

    print('模型docker镜像 {}:{} 构建完成！ '.format(image_name, image_tag))


if __name__ == '__main__':
    pack_model()



import json
import yaml
import getpass
import requests
from urllib.parse import quote
from urllib3 import encode_multipart_formdata


def get_tokens(url_server):
    url_login = '{}/api/data'.format(url_server)

    print('即将向CubeAI平台推送模型并生成微服务...')
    print('请输入您在CubeAI平台注册的用户名和口令：\n')
    username = input('username: ')
    password = getpass.getpass('password: ')

    body = {
        'action': 'login_cmd',
        'args': {
            'username': username,
            'password': password,
        }
    }
    res = requests.post(url_login, json=body)
    res = json.loads(res.text, encoding='utf-8')

    if res['status'] == 'ok':
        tokens = res['value']
        return 'refresh_token={}; access_token={}'.format(tokens['refresh_token'], tokens['access_token'])
    else:
        return None


def get_file_name():
    try:
        with open('./application.yml', 'r') as f:
            yml = yaml.load(f, Loader=yaml.SafeLoader)
    except:
        print('错误： 模型配置文件application.yml不存在！')
        return None

    try:
        name = yml['model']['name']
    except:
        print('错误： 未指定模型名称！')
        print('请在application.yml文件中编辑修改...')
        return None

    return '{}.zip'.format(name)


def onboarding(url_server='https://cubeai.dimpt.com'):
    file_name = get_file_name()
    if file_name is None:
        return

    tokens = get_tokens(url_server)
    if tokens is None:
        print('用户名或密码错误！')
        return

    data = {
        'onboard_model': (file_name, open('out/{}'.format(file_name), 'rb').read())
    }
    encode_data = encode_multipart_formdata(data)

    headers = {
        'Content-Type': encode_data[1],
        'Cookie': tokens,
    }

    url_onboarding = '{}/umu/api/file/onboard_model'.format(url_server)
    res = requests.post(url_onboarding, headers=headers, data=encode_data[0])
    res = json.loads(res.text, encoding='utf-8')
    if res['status'] == 'ok':

        task_uuid = res['value']
        url_task = '{}/pmodelhub/#/task-onboarding/{}/{}'.format(url_server, task_uuid, quote(file_name, 'utf-8'))
        print('文件上传成功！正在生成微服务...')
        print('请打开CubeAI平台任务详情页面查看模型导入进度： {} \n'.format(url_task))
    else:
        print('文件上传失败： {}'.format(res['value']))


if __name__ == '__main__':
    import sys
    if len(sys.argv) > 1:
        onboarding(sys.argv[1])
    else:
        onboarding()



# -*- coding: utf-8 -*-
import json
import socket
import threading
import logging
import asyncio
import tornado.web
import tornado.ioloop
import tornado.httpserver
import tornado.websocket

import os, sys
sys.path.append(os.getcwd())
from core.model_core import ModelCore


class GlobalData:
    def __init__(self):
        self.model_core = None

    def init_global_data(self):
        try:
            self.model_core = ModelCore()
            return True
        except Exception as e:
            logging.error(str(e))
            return False


g = GlobalData()


class ModelApi(tornado.web.RequestHandler):

    async def post(self, *args, **kwargs):

        try:
            input = json.loads(str(self.request.body, encoding='utf-8'))
        except Exception as e:
            logging.error(str(e))
            result = {
                'status': 'err',
                'value': '错误： HTTP请求体错误！' + str(e)
            }
            res = json.dumps(result, ensure_ascii=False)
            self.set_header('Content-Type', 'application/json; charset=UTF-8')
            self.write(res)
            return 
            
        output = {
            'result': {},
            'finish': False
        }
        thread = threading.Thread(
            target=model_runner,
            args=(input, output)
        )
        thread.setDaemon(True)
        thread.start()

        while not output['finish']:
            await asyncio.sleep(0.05)

        result = output['result']

        if isinstance(result['value'], bytes):
            res = result['value']
        else:
            try:
                res = json.dumps(result, ensure_ascii=False)
            except Exception as e:
                logging.error(str(e))
                result = {
                    'status': 'err',
                    'value': '错误： 模型返回结果序列化失败！' + str(e)
                }
                res = json.dumps(result, ensure_ascii=False)
            self.set_header('Content-Type', 'application/json; charset=UTF-8')

        if self.request.headers.get('Origin'):
            self.set_header('Access-Control-Allow-Credentials', 'true')
            self.set_header('Access-Control-Allow-Origin', self.request.headers.get('Origin'))
        self.write(res)

    async def options(self, *args, **kwargs):
        self.set_status(204)
        self.set_header('Access-Control-Allow-Credentials', 'true')
        self.set_header('Access-Control-Allow-Origin', self.request.headers.get('Origin'))
        self.set_header('Access-Control-Allow-Headers', 'content-type')
        self.set_header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS')
        self.finish()
        return


def model_runner(input, output):
    if not isinstance(input, dict):
        result = {
            'value': '错误： HTTP请求体不是JSON格式！',
            'status': 'err'
        }
        output['result'] = result
        output['finish'] = True
        return

    action = input.get('action')
    kwargs = input.get('args')

    if action is None:
        result = {
            'value': '错误： HTTP请求体中未携带方法名！',
            'status': 'err'
        }
        output['result'] = result
        output['finish'] = True
        return

    public_methods = getattr(ModelCore, 'public_methods', None)
    if public_methods is not None:
        if action not in public_methods:
            result = {
                'value': '错误： 方法 {} 禁止访问！'.format(action),
                'status': 'err'
            }
            output['result'] = result
            output['finish'] = True
            return

    model = g.model_core
    method_obj = getattr(model, action, None)
    if method_obj is None:
        result = {
            'value': '错误： 方法 {} 未定义！'.format(action),
            'status': 'err'
        }
        output['result'] = result
        output['finish'] = True
        return

    result = {}
    try:
        result['value'] = method_obj(**kwargs) if kwargs else method_obj()  # kwargs为None或{}时，不带参数调用
        result['status'] = 'ok'
    except Exception as e:
        logging.error(str(e))
        result['value'] = str(e)
        result['status'] = 'err'

    output['result'] = result
    output['finish'] = True


class StreamApi(tornado.web.RequestHandler):

    async def post(self, action, *args, **kwargs):

        input = {
            'action': action,
            'arg': self.request.body,
        }
        output = {
            'result': {},
            'finish': False
        }
        thread = threading.Thread(
            target=stream_runner,
            args=(input, output)
        )
        thread.setDaemon(True)
        thread.start()

        while not output['finish']:
            await asyncio.sleep(0.05)

        result = output['result']

        if isinstance(result['value'], bytes):
            res = result['value']
        else:
            try:
                res = json.dumps(result, ensure_ascii=False)
            except Exception as e:
                logging.error(str(e))
                result = {
                    'status': 'err',
                    'value': '错误： 模型返回结果序列化失败！' + str(e)
                }
                res = json.dumps(result, ensure_ascii=False)
            self.set_header('Content-Type', 'application/json; charset=UTF-8')

        if self.request.headers.get('Origin'):
            self.set_header('Access-Control-Allow-Credentials', 'true')
            self.set_header('Access-Control-Allow-Origin', self.request.headers.get('Origin'))
        self.write(res)

    async def options(self, *args, **kwargs):
        self.set_status(204)
        self.set_header('Access-Control-Allow-Credentials', 'true')
        self.set_header('Access-Control-Allow-Origin', self.request.headers.get('Origin'))
        self.set_header('Access-Control-Allow-Headers', 'content-type')
        self.set_header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS')
        self.finish()
        return


def stream_runner(input, output):

    action = input.get('action')
    arg = input.get('arg')

    public_methods = getattr(ModelCore, 'public_methods', None)
    if public_methods is not None:
        if action not in public_methods:
            result = {
                'value': '错误： 方法 {} 禁止访问！'.format(action),
                'status': 'err'
            }
            output['result'] = result
            output['finish'] = True
            return

    model = g.model_core
    method_obj = getattr(model, action, None)
    if method_obj is None:
        result = {
            'value': '错误： 方法 {} 未定义！'.format(action),
            'status': 'err'
        }
        output['result'] = result
        output['finish'] = True
        return

    result = {}
    try:
        result['value'] = method_obj(arg)
        result['status'] = 'ok'
    except Exception as e:
        logging.error(str(e))
        result['value'] = str(e)
        result['status'] = 'err'

    output['result'] = result
    output['finish'] = True


class FileApi(tornado.web.RequestHandler):

    async def post(self, action, *args, **kwargs):
        
        try:
            file_obj = self.request.files.get(action)[0]
            file_body = file_obj.body
        except Exception as e:
            logging.error(str(e))
            result = {
                'status': 'err',
                'value': '错误： HTTP文件请求体错误！' + str(e)
            }
            res = json.dumps(result, ensure_ascii=False)
            self.set_header('Content-Type', 'application/json; charset=UTF-8')
            self.write(res)
            return 

        input = {
            'action': action,
            'arg': file_body,
        }
        output = {
            'result': {},
            'finish': False
        }
        thread = threading.Thread(
            target=stream_runner,
            args=(input, output)
        )
        thread.setDaemon(True)
        thread.start()

        while not output['finish']:
            await asyncio.sleep(0.05)

        result = output['result']

        if isinstance(result['value'], bytes):
            res = result['value']
        else:
            try:
                res = json.dumps(result, ensure_ascii=False)
            except Exception as e:
                logging.error(str(e))
                result = {
                    'status': 'err',
                    'value': '错误： 模型返回结果序列化失败！' + str(e)
                }
                res = json.dumps(result, ensure_ascii=False)
            self.set_header('Content-Type', 'application/json; charset=UTF-8')

        if self.request.headers.get('Origin'):
            self.set_header('Access-Control-Allow-Credentials', 'true')
            self.set_header('Access-Control-Allow-Origin', self.request.headers.get('Origin'))
        self.write(res)

    async def options(self, *args, **kwargs):
        self.set_status(204)
        self.set_header('Access-Control-Allow-Credentials', 'true')
        self.set_header('Access-Control-Allow-Origin', self.request.headers.get('Origin'))
        self.set_header('Access-Control-Allow-Headers', 'content-type')
        self.set_header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS')
        self.finish()
        return
    

class WebSocketServer(tornado.websocket.WebSocketHandler):

    def open(self):
        pass

    def on_message(self, message):
        try:
            msg = json.loads(message, encoding='utf-8')
        except Exception as e:
            logging.error(str(e))
            logging.error('WebSocket消息必须采用JSON格式！')
            return

        thread = threading.Thread(
            target=websocket_message_runner,
            args=(asyncio.get_event_loop(), self, msg)
        )
        thread.setDaemon(True)
        thread.start()

    def on_close(self):
        logging.critical('websocket closed')

    def check_origin(self, origin):
        return True


def websocket_message_runner(event_loop, websocket, msg):
    asyncio.set_event_loop(event_loop)
    try:
        g.model_core.process_websocket_message(websocket, msg)
    except Exception as e:
        logging.error(str(e))



def get_local_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()

    return ip


def start():
    logging.basicConfig(level=logging.ERROR, format='%(asctime)s - %(levelname)s - %(message)s')

    if not g.init_global_data():
        logging.error('错误： 模型初始化失败！')
        return

    app_profile = os.environ.get('APP_PROFILE', 'dev').lower()
    app = tornado.web.Application(
        handlers=[
            (r'/api/model', ModelApi),
            (r'/api/stream/(\w+)', StreamApi),
            (r'/api/file/(\w+)', FileApi),
            (r'/web/(.*)', tornado.web.StaticFileHandler, {'path': 'webapp/www', 'default_filename': 'index.html'}),
            (r'/websocket', WebSocketServer),
        ],
        debug=('dev' == app_profile)
    )
    http_server = tornado.httpserver.HTTPServer(app, max_buffer_size=800*1024*1024)
    http_server.listen(3330)

    logging.critical('##################################################')
    logging.critical('    AI-Boot started ...')
    logging.critical('    Listening at: {}:3330'.format(get_local_ip()))
    logging.critical('    Web access: http://{}:3330/web/'.format(get_local_ip()))
    logging.critical('    App profile: {}'.format(app_profile))
    logging.critical('##################################################')
    tornado.ioloop.IOLoop.current().start()


if __name__ == '__main__':
    start()
